#!/bin/bash
for demo in code_server custom_apt_key_dearmor jupyterlab/v3 jupyterlab/v4 mariadb openrefine tutorial_server
do
    ./demo-tests.sh $demo
    if [ ! $? == "0" ]
    then
        exit 1
    fi
done
